package cz.literak.blog.j2ee;

import oracle.sql.ARRAY;
import oracle.sql.ArrayDescriptor;
import oracle.sql.STRUCT;
import oracle.sql.StructDescriptor;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.annotation.Resource;
import javax.ejb.Stateless;
import java.sql.*;

/**
 * Example session bean that can get and send very complex oracle types.
 * Date: 14.8.13
 * @author Leos Literak
 */
@Stateless(mappedName = "test/oracle")
public class OracleTest implements IOracleTest {
    private static final Log log = LogFactory.getLog(OracleTest.class);

    @Resource(name = "jdbc/test")
    protected javax.sql.DataSource datasource;

    public void performGet() throws Exception {
        String sql = "{ ? = call oracle_types.get()}";
        CallableStatement cs = null;
        Connection conn = null;

        try {
            conn = datasource.getConnection();
            cs = conn.prepareCall(sql);

            // register structs used in the query
            StructDescriptor keywordStructDesc = StructDescriptor.createDescriptor("KEYWORD_REC", conn);
            StructDescriptor personStructDesc = StructDescriptor.createDescriptor("PERSON_REC", conn);
            ResultSetMetaData metaData = personStructDesc.getMetaData();
            cs.registerOutParameter(1, Types.STRUCT, "PERSON_REC");
            cs.execute();

            STRUCT output;
            Object recordTmp = cs.getObject(1);
            if (recordTmp instanceof Struct){
                output = (oracle.sql.STRUCT)(((weblogic.jdbc.wrapper.Struct)recordTmp).unwrap(Class.forName("oracle.sql.STRUCT")));
            } else {
                output = (oracle.sql.STRUCT)recordTmp;
            }

            Object[] data = output.getAttributes();
            int idx = 1;
            for (Object tmp : data) {
                // orai8n.jar must be in weblogic classpath for strings
                log.info(metaData.getColumnName(idx++) + " = " + tmp + ", class=" + tmp.getClass().getName());
                log.info("---");
		    }
        } catch (Exception e) {
            log.error("Chyba", e);
            throw new Exception(e.getMessage());
        } finally {
            if (cs != null) {
                cs.close();
            }
            if (conn != null) {
                conn.close();
            }
        }
    }

    public void performCall() throws Exception {
        String sql = "{ ? = call oracle_types.add(?, ?)}";
        CallableStatement prepStmt = null;
        Connection conn = null;

        try {
            conn = datasource.getConnection();

            // register structs used in the query
            ArrayDescriptor keywordsArrayDesc = ArrayDescriptor.createDescriptor("KEYWORDS_REC", conn);
            StructDescriptor personStructDesc = StructDescriptor.createDescriptor("PERSON_REC", conn);

            Object[][] keywordsAttribs = new Object[2][1];
            keywordsAttribs[0][0] = "actor";
            keywordsAttribs[1][0] = "producer";

            Object[] personAttribs = new Object[4];
            personAttribs[0] = "Christian";
            personAttribs[1] = "Bale";
            personAttribs[2] = 39;
            personAttribs[3] = new ARRAY(keywordsArrayDesc, conn, keywordsAttribs);
            STRUCT struct = new STRUCT(personStructDesc, conn, personAttribs);

            prepStmt = conn.prepareCall(sql);
            prepStmt.registerOutParameter(1, Types.INTEGER);
            prepStmt.registerOutParameter(3, Types.VARCHAR);
            prepStmt.setObject(2, struct);

            prepStmt.execute();
            int returnCode = prepStmt.getInt(1);
            String message = prepStmt.getString(3);
            log.info("returnCode=" + returnCode + ", message=" + message);
        } catch (Exception e) {
            log.error("Chyba", e);
            throw new Exception(e.getMessage());
        } finally {
            if (prepStmt != null) {
                prepStmt.close();
            }
            if (conn != null) {
                conn.close();
            }
        }
    }
}
