BEGIN
   EXECUTE IMMEDIATE 'DROP TABLE PERSON';
EXCEPTION
   WHEN OTHERS THEN
      IF SQLCODE != -942 THEN
         RAISE;
      END IF;
end;
/

CREATE TABLE PERSON (
    name    VARCHAR2(20),
    surname VARCHAR2(30),
    age     NUMBER(3),
    vip     CHAR(1)
);

CREATE OR REPLACE TYPE keyword_rec IS OBJECT (
    value VARCHAR2(20)
);

CREATE OR REPLACE TYPE keywords_rec IS TABLE OF keyword_rec;

CREATE OR REPLACE TYPE person_rec IS OBJECT (
    name     VARCHAR2(20),
    surname  VARCHAR2(30),
	age      NUMBER(3),
    keywords keywords_rec
);

CREATE OR REPLACE PACKAGE oracle_types AS
  FUNCTION add(p_rec person_rec, p_message OUT VARCHAR2) RETURN PLS_INTEGER;
  FUNCTION get RETURN person_rec;
end oracle_types;
/

CREATE OR REPLACE PACKAGE BODY oracle_types IS
  FUNCTION add(p_rec person_rec, p_message out varchar2) RETURN PLS_INTEGER IS
   v_vip NUMBER := 0;
  BEGIN
    FOR cur IN 1 .. p_rec.keywords.LAST
    LOOP
      IF p_rec.keywords(cur).value = 'actor' THEN
        v_vip := 1;
      END IF;
    END LOOP;
      IF v_vip > 0 THEN
        INSERT INTO PERSON(name,surname,age,vip) VALUES (p_rec.name, p_rec.surname, p_rec.age, '1');
      ELSE
        INSERT INTO PERSON(name,surname,age,vip) VALUES (p_rec.name, p_rec.surname, p_rec.age, '0');
      END IF;
   RETURN 1;
  END;

  FUNCTION get RETURN person_rec IS
   v_person person_rec;
   v_keywords keywords_rec;
  begin
    v_keywords := keywords_rec();
    v_keywords.EXTEND(2);
    v_keywords(1) := keyword_rec('scientist');
    v_keywords(2) := keyword_rec('inventor');
    v_person := person_rec('Antonin', 'Holy', 75, v_keywords);
    RETURN v_person;
  END;

BEGIN
  NULL;
end oracle_types;
/

declare
 v_result person_rec;
begin
  v_result := oracle_types.get();
  dbms_output.put_line('name = '||v_result.name);
  dbms_output.put_line('surname = '||v_result.surname);
  dbms_output.put_line('age = '||v_result.age);
  for cur in 1 .. v_result.keywords.last
  loop
    dbms_output.put_line('keyword = '||v_result.keywords(cur).value);
  end loop;
end;
/
